from fundb.sqlalchemy.table import BaseTable
from funutil import getLogger
from sqlalchemy import String
from sqlalchemy.orm import Mapped, mapped_column

logger = getLogger("funsketch")


class Episode(BaseTable):
    __tablename__ = "episode"
    sketch_id: Mapped[str] = mapped_column(String(64), comment="文件唯一ID", default="")
    index: Mapped[int] = mapped_column(comment="集", default="")
    name: Mapped[str] = mapped_column(String(128), comment="资源名称")
    size: Mapped[int] = mapped_column(comment="大小", default=0)
    fid: Mapped[str] = mapped_column(String(64), comment="文件唯一ID", default="")
    text: Mapped[str] = mapped_column(String(1200), comment="视频文本", default="")

    def _get_uid(self):
        return f"{self.sketch_id}:{self.index}"

    def _child(self):
        return Episode

    def _to_dict(self) -> dict:
        return {
            "sketch_id": self.sketch_id,
            "index": self.index,
            "name": self.name,
            "size": self.size,
            "fid": self.fid,
            "text": self.text,
        }



import json

from funai.llm import get_model
from fundb.sqlalchemy.table import BaseTable
from fundrive.drives.baidu.drive import BaiDuDrive
from funsecret import read_secret
from funutil import getLogger
from sqlalchemy import Engine
from sqlalchemy.orm import Session

from funsketch.db import Episode, Sketch

logger = getLogger("funsketch")

model = get_model("deepseek")


def sync_episode_from_sketch(
    engine: Engine, bduss=None, stoken=None, ptoken=None, *args, **kwargs
):
    bduss = bduss or read_secret("fundrive", "baidu", "bduss")
    stoken = stoken or read_secret("fundrive", "baidu", "stoken")
    ptoken = ptoken or read_secret("fundrive", "baidu", "ptoken")
    drive = BaiDuDrive()
    drive.login(bduss=bduss, stoken=stoken, ptoken=ptoken)

    with Session(engine) as session:
        res = BaseTable.select_all(session=session, table=Sketch)
        for sketch in res:
            print(sketch.uid)
            data = [
                data["fid"]
                for data in drive.get_file_list(sketch.fid)
                if data["fid"].endswith(".mp4")
            ]
            data = "\n".join([i for i in data])
            prompt = f"""
                下面是一部电视剧的所有文件路径，请分析这些文件名，返回视频顺序，要求返回结果是json的列表，包含path，index, name,只返回json，不要用```包含
                {data}
                """
            res = model.chat(prompt)
            res = json.loads(res)

            for data in res:
                episode = Episode(
                    fid=data["path"],
                    name=data["name"],
                    index=data["index"],
                    sketch_id=sketch.uid,
                )
                episode.upsert(session=session)
            session.commit()



from fundrive.core import BaseDrive
from fundrive.drives.alipan import AlipanDrive
from funsecret import read_secret
from funsketch.db import Sketch
from funutil import getLogger
from sqlalchemy import create_engine
from sqlalchemy.orm import Session

logger = getLogger("funsketch")


def sync_data(
    driver: BaseDrive = None,
    sketch_fid="677a4bbc22c22185a94449fea7d7526e0905696d",
    funsketch_fid="677a78422552250db76c412c98b2c1d9b2779a58",
):
    if driver is None:
        driver = AlipanDrive()
        driver.login(is_resource=True)

    url = read_secret("funsketch", "db", "url")
    engine = create_engine(url, echo=False)
    with Session(engine) as session:
        for file in driver.get_dir_list(sketch_fid):
            fid = driver.mkdir(fid=funsketch_fid, name=file["name"])
            sketch = Sketch(fid=fid, name=file["name"])
            sketch.upsert(session)
            session.commit()



from fundb.sqlalchemy.table import BaseTable
from funutil import getLogger
from sqlalchemy import Engine, String
from sqlalchemy.orm import Mapped, Session, mapped_column

logger = getLogger("funsketch")


class Sketch(BaseTable):
    __tablename__ = "sketch"
    name: Mapped[str] = mapped_column(String(128), comment="资源名称")
    fid: Mapped[str] = mapped_column(String(64), comment="文件唯一ID", default="")

    def _get_uid(self):
        return self.fid

    def _child(self):
        return Sketch

    def _to_dict(self):
        return {"name": self.name, "fid": self.fid}


def add_sketch(engine: Engine, name, fid):
    BaseTable.metadata.create_all(engine)
    with Session(engine) as session:
        Sketch(name=name, fid=fid).upsert(session=session, update_data=True)
        session.commit()



import json
import os

from funtalk.asr import WhisperASR
from funutil import getLogger
from moviepy import VideoFileClip
from sqlalchemy import Engine, func, select
from sqlalchemy.orm import Session

from funsketch.db import Episode

logger = getLogger("funsketch")


class EpisodePath:
    def __init__(self, episode: Episode):
        self.episode = episode
        self.sketch_dir = f"funsketch/{episode.sketch_id}"

        self.video_path = (
            f"{self.sketch_dir}/{str(episode.index).zfill(3)}-{episode.uid}.mp4"
        )
        self.audio_path = (
            f"{self.sketch_dir}/{str(episode.index).zfill(3)}-{episode.uid}.wav"
        )
        self.text_path = (
            f"{self.sketch_dir}/{str(episode.index).zfill(3)}-{episode.uid}.txt"
        )
        os.makedirs(self.sketch_dir, exist_ok=True)

    def download_video(self, driver):
        driver.download_file(
            self.episode.fid,
            local_dir=self.sketch_dir,
            filepath=self.video_path,
            overwrite=False,
        )

    def convert_video(self, *args, **kwargs):
        if os.path.exists(self.audio_path):
            logger.info(f"audio file {self.audio_path} already exists")
            return
        video_clip = VideoFileClip(self.video_path)
        audio_clip = video_clip.audio
        audio_clip.write_audiofile(self.audio_path)
        audio_clip.close()
        video_clip.close()

    def detect_text(self):
        if os.path.exists(self.text_path):
            logger.info(f"text file {self.text_path} already exists")
            return
        model = WhisperASR("turbo")
        result = model.transcribe(self.audio_path, language="zh")
        with open(self.text_path, "w", encoding="utf-8") as f:
            f.write(json.dumps(result))
        logger.success(f"{self.text_path} success")


def update_episode(engine: Engine, drive, *args, **kwargs):
    with Session(engine) as session:
        episodes = session.execute(
            select(Episode).where(func.char_length(Episode.text) < 10)
        ).scalars()

        for episode in episodes:
            episode_path = EpisodePath(episode)
            episode_path.download_video(driver=drive)
            episode_path.convert_video()
            episode_path.detect_text()
            text_path = f"/{episode_path.text_path}"
            drive.upload_file(local_path=episode_path.text_path, fid=text_path)
            episode.text = text_path
            episode.upsert(session=session)
            session.commit()
